@@ -46,12 +46,12 @@ enum {
 static GParamSpec *obj_properties[LAST_PROP];
 
 typedef struct {
-  EphyHistoryManager                     *manager;
-  gboolean                                is_initial;
-  GList                                  *remotes_deleted;
-  GList                                  *remotes_updated;
-  EphySynchronizableManagerMergeCallback  callback;
-  gpointer                                user_data;
+  EphyHistoryManager *manager;
+  gboolean is_initial;
+  GList *remotes_deleted;
+  GList *remotes_updated;
+  EphySynchronizableManagerMergeCallback callback;
+  gpointer user_data;
 } MergeHistoryAsyncData;
 
 static MergeHistoryAsyncData *
@@ -188,8 +188,7 @@ ephy_history_manager_class_init (EphyHistoryManagerClass *klass)
 
   obj_properties[PROP_HISTORY_SERVICE] =
     g_param_spec_object ("history-service",
-                         "History service",
-                         "History Service",
+                         NULL, NULL,
                          EPHY_TYPE_HISTORY_SERVICE,
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
@@ -364,7 +363,7 @@ ephy_history_manager_handle_initial_merge (EphyHistoryManager *self,
 
       g_hash_table_remove (records_ht_id, remote_id);
     } else {
-    /* Try find by URL. */
+      /* Try find by URL. */
       record = g_hash_table_lookup (records_ht_url, remote_url);
       if (record) {
         /* Different ID, same URL. Keep local ID. */
@@ -391,11 +390,11 @@ ephy_history_manager_handle_initial_merge (EphyHistoryManager *self,
 }
 
 static GPtrArray *
-ephy_history_manager_handle_regular_merge (EphyHistoryManager  *self,
-                                           GHashTable          *records_ht_id,
-                                           GHashTable          *records_ht_url,
-                                           GList               *deleted_records,
-                                           GList               *updated_records)
+ephy_history_manager_handle_regular_merge (EphyHistoryManager *self,
+                                           GHashTable         *records_ht_id,
+                                           GHashTable         *records_ht_url,
+                                           GList              *deleted_records,
+                                           GList              *updated_records)
 {
   EphyHistoryRecord *record;
   GPtrArray *to_upload;
@@ -513,7 +512,6 @@ merge_history_cb (EphyHistoryService    *service,
 out:
   data->callback (to_upload, data->user_data);
 
-  g_list_free_full (urls, (GDestroyNotify)ephy_history_url_free);
   if (records_ht_id)
     g_hash_table_unref (records_ht_id);
   if (records_ht_url)
