@@ -34,15 +34,14 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <webkit2/webkit2.h>
+#include <webkit/webkit.h>
 
 struct _EphyEncodingDialog {
-  GtkDialog parent_instance;
+  GtkWindow parent_instance;
 
   EphyEncodings *encodings;
   EphyWindow *window;
   EphyEmbed *embed;
-  GtkWidget *enc_view;
   gboolean update_embed_tag;
   gboolean update_view_tag;
   const char *selected_encoding;
@@ -53,8 +52,8 @@ struct _EphyEncodingDialog {
   GtkListBox *list_box;
   GtkListBox *recent_list_box;
   GtkListBox *related_list_box;
-  GtkGrid *recent_grid;
-  GtkGrid *related_grid;
+  GtkWidget *recent_box;
+  GtkWidget *related_box;
 };
 
 enum {
@@ -71,37 +70,36 @@ enum {
 
 static GParamSpec *obj_properties[LAST_PROP];
 
-G_DEFINE_TYPE (EphyEncodingDialog, ephy_encoding_dialog, GTK_TYPE_DIALOG)
+G_DEFINE_TYPE (EphyEncodingDialog, ephy_encoding_dialog, GTK_TYPE_WINDOW)
 
 static void
 select_encoding_row (GtkListBox   *list_box,
                      EphyEncoding *encoding)
 {
   const char *target_encoding;
-  GList *rows, *r;
+  GtkListBoxRow *row;
+  int i = 0;
 
   target_encoding = ephy_encoding_get_encoding (encoding);
-  rows = gtk_container_get_children (GTK_CONTAINER (list_box));
 
-  for (r = rows; r != NULL; r = r->next) {
+  while ((row = gtk_list_box_get_row_at_index (list_box, i++))) {
     EphyEncodingRow *ephy_encoding_row;
     EphyEncoding *ephy_encoding;
     const char *encoding_string = NULL;
 
-    ephy_encoding_row = EPHY_ENCODING_ROW (gtk_bin_get_child (GTK_BIN (r->data)));
+    ephy_encoding_row = EPHY_ENCODING_ROW (gtk_list_box_row_get_child (row));
     ephy_encoding = ephy_encoding_row_get_encoding (ephy_encoding_row);
     encoding_string = ephy_encoding_get_encoding (ephy_encoding);
 
     if (g_strcmp0 (encoding_string, target_encoding) == 0) {
       ephy_encoding_row_set_selected (ephy_encoding_row, TRUE);
 
-      gtk_list_box_select_row (list_box, GTK_LIST_BOX_ROW (r->data));
+      gtk_list_box_select_row (list_box, row);
       /* TODO scroll to row */
 
       break;
     }
   }
-  g_list_free (rows);
 }
 
 static void
@@ -213,30 +211,17 @@ activate_choice (EphyEncodingDialog *dialog)
   }
 }
 
-static void
-ephy_encoding_dialog_response_cb (GtkWidget          *widget,
-                                  int                 response,
-                                  EphyEncodingDialog *dialog)
-{
-  gtk_widget_destroy (GTK_WIDGET (dialog));
-}
-
-static void
-clean_selected_row (gpointer row,
-                    gpointer null_pointer)
-{
-  EphyEncodingRow *ephy_encoding_row;
-  ephy_encoding_row = EPHY_ENCODING_ROW (gtk_bin_get_child (GTK_BIN (row)));
-  ephy_encoding_row_set_selected (ephy_encoding_row, FALSE);
-}
-
 static void
 clean_selected_list_box (GtkListBox *list_box)
 {
-  GList *rows;
-  rows = gtk_container_get_children (GTK_CONTAINER (list_box));
-  g_list_foreach (rows, (GFunc)clean_selected_row, NULL);
-  g_list_free (rows);
+  GtkListBoxRow *row;
+  int i = 0;
+
+  while ((row = gtk_list_box_get_row_at_index (list_box, i++))) {
+    EphyEncodingRow *ephy_encoding_row =
+      EPHY_ENCODING_ROW (gtk_list_box_row_get_child (GTK_LIST_BOX_ROW (row)));
+    ephy_encoding_row_set_selected (ephy_encoding_row, FALSE);
+  }
 }
 
 static void
@@ -261,7 +246,7 @@ row_activated_cb (GtkListBox         *box,
 
   dialog->update_view_tag = TRUE;
 
-  ephy_encoding_row = EPHY_ENCODING_ROW (gtk_bin_get_child (GTK_BIN (row)));
+  ephy_encoding_row = EPHY_ENCODING_ROW (gtk_list_box_row_get_child (row));
   ephy_encoding = ephy_encoding_row_get_encoding (ephy_encoding_row);
   selected_encoding = ephy_encoding_get_encoding (ephy_encoding);
 
@@ -329,7 +314,7 @@ static void
 add_list_item (EphyEncoding *encoding,
                GtkListBox   *list_box)
 {
-  gtk_container_add (GTK_CONTAINER (list_box), GTK_WIDGET (ephy_encoding_row_new (encoding)));
+  gtk_list_box_append (GTK_LIST_BOX (list_box), GTK_WIDGET (ephy_encoding_row_new (encoding)));
 }
 
 static int
@@ -399,7 +384,7 @@ ephy_encoding_dialog_constructed (GObject *object)
     recent = g_list_sort (recent, (GCompareFunc)sort_encodings);
     g_list_foreach (recent, (GFunc)add_list_item, dialog->recent_list_box);
   } else
-    gtk_widget_hide (GTK_WIDGET (dialog->recent_grid));
+    gtk_widget_set_visible (dialog->recent_box, FALSE);
 
   /* related */
   if (dialog->selected_encoding != NULL) {
@@ -413,7 +398,7 @@ ephy_encoding_dialog_constructed (GObject *object)
     related = g_list_sort (related, (GCompareFunc)sort_encodings);
     g_list_foreach (related, (GFunc)add_list_item, dialog->related_list_box);
   } else
-    gtk_widget_hide (GTK_WIDGET (dialog->related_grid));
+    gtk_widget_set_visible (dialog->related_box, FALSE);
 
   /* update list_boxes */
   sync_encoding_against_embed (dialog);
@@ -498,8 +483,7 @@ ephy_encoding_dialog_class_init (EphyEncodingDialogClass *klass)
 
   obj_properties[PROP_PARENT_WINDOW] =
     g_param_spec_object ("parent-window",
-                         "Parent window",
-                         "Parent window",
+                         NULL, NULL,
                          EPHY_TYPE_WINDOW,
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
@@ -513,20 +497,20 @@ ephy_encoding_dialog_class_init (EphyEncodingDialogClass *klass)
   gtk_widget_class_bind_template_child (widget_class, EphyEncodingDialog, list_box);
   gtk_widget_class_bind_template_child (widget_class, EphyEncodingDialog, recent_list_box);
   gtk_widget_class_bind_template_child (widget_class, EphyEncodingDialog, related_list_box);
-  gtk_widget_class_bind_template_child (widget_class, EphyEncodingDialog, recent_grid);
-  gtk_widget_class_bind_template_child (widget_class, EphyEncodingDialog, related_grid);
+  gtk_widget_class_bind_template_child (widget_class, EphyEncodingDialog, recent_box);
+  gtk_widget_class_bind_template_child (widget_class, EphyEncodingDialog, related_box);
 
   gtk_widget_class_bind_template_callback (widget_class, default_switch_toggled_cb);
-  gtk_widget_class_bind_template_callback (widget_class, ephy_encoding_dialog_response_cb);
   gtk_widget_class_bind_template_callback (widget_class, row_activated_cb);
   gtk_widget_class_bind_template_callback (widget_class, show_all_button_clicked_cb);
+
+  gtk_widget_class_add_binding_action (widget_class, GDK_KEY_Escape, 0, "window.close", NULL);
 }
 
 EphyEncodingDialog *
 ephy_encoding_dialog_new (EphyWindow *parent)
 {
   return g_object_new (EPHY_TYPE_ENCODING_DIALOG,
-                       "use-header-bar", TRUE,
                        "parent-window", parent,
                        NULL);
 }
